#ifndef KSANEEXTENDEDWIDGET_H
#define KSANEEXTENDEDWIDGET_H

#include <QStackedWidget>
#include <QList>

#include "ksanepage.h"

#include "ksane_export.h"

class DocumentSaver;

namespace KSaneIface
{

class AutoScanWidget;

class KSaneWidget;

class KSaneExtendedWidgetPrivate;

class KSANE_EXPORT KSaneExtendedWidget : public QStackedWidget
{
    Q_OBJECT

public:
    enum class Mode {
        AutoScan = 0,
        ManualScan
    };

    typedef enum {
        Good = 0,
        NoDocs = 7
    } SaneStatus;

    struct DeviceInfo {
        QString name;     /* unique device name */
        QString vendor;   /* device vendor string */
        QString model;    /* device model name */
        QString type;     /* device type (e.g., "flatbed scanner") */
        QString label;
    };

    KSaneExtendedWidget(QWidget *parent = nullptr);
    ~KSaneExtendedWidget();

    /** This helper method displays a dialog for selecting a scanner. The libsane
     * device name of the selected scanner device is returned. */
    DeviceInfo selectDevice(QWidget *parent = nullptr);

    /**
     * Get the list of available scanning devices. Connect to availableDevices()
     * which is fired once these devices are known.
     */
    void initGetDeviceList() const;

    /** This method opens the specified scanner device and adds the scan options to the
     * KSane widget.
     * @param device_name is the libsane device name for the scanner to open.
     * @return 'true' if all goes well and 'false' if the specified scanner can not be opened. */
    bool openDevice(const QString &device_name, const QString &device_label = QLatin1String(""));

    /** This method closes the currently open scanner device.
    * @return 'true' if all goes well and 'false' if no device is open. */
    bool closeDevice();

    /** This method returns the current resolution of the acquired image,
    * in dots per inch.
    * @note This function should be called from the slot connected
    * to the imageReady signal. The connection should not be queued.
    * @return the resolution used for scanning or 0.0 on failure. */
    float currentDPI();

    bool isExecuteMultiPageScanning();

    void setMode(KSaneExtendedWidget::Mode mode);
    KSaneExtendedWidget::Mode mode();

    bool isAutoScan();

    QMap<QString, QMap<QString, QString>> templateParam();
    void setTemplateParam(const QMap<QString, QMap<QString, QString>> &param);

    QList<DocumentSaver *> documentSavers();

    QString imageFormats();
    void setMultipageFormats(const QStringList &formats);

    void setSavePath(const QString &path);
    QString savePath();

    void setDeviceLabel(const QString &deviceLabel);

    QStringList scanModes();
    QStringList scanResolutions();    

    QString currentFormat();

    /** This method sets the selection according to the given points
    * @note The points are defined with respect to the scan areas top-left corner in mm
    * @param topLeft Upper left corner of the selection (in mm)
    * @param bottomRight Lower right corner of the selection (in mm) */
    void setSelection(QPointF topLeft, QPointF bottomRight);

    /** This function is used to set the preferred resolution for scanning the preview.
    * @param dpi is the wanted scan resolution for the preview
    * @note if the set value is not supported, the cloasest one is used
    * @note setting the value 0 means that the default calculated value should be used */
    void setPreviewResolution(float dpi);

    /** This method reads the available parameters and their values and
     * returns them in a QMap (Name, value)
     * @param opts is a QMap with the parameter names and values. */
    void getOptVals(QMap <QString, QString> &opts);

    /** This method can be used to write many parameter values at once.
     * @param opts is a QMap with the parameter names and values.
     * @return This function returns the number of successful writes
     * or -1 if scanning is in progress. */
    int setOptVals(const QMap <QString, QString> &opts);    

    void setManyFilesForMultipage(bool manyFiles);
    bool manyFilesForMultipage();    

    void setCurrentImageFormat(KSaneExtendedWidget::Mode mode, const QString &format);
    QString currentImageFormat(KSaneExtendedWidget::Mode mode);

    void setCurrentTextFormat(KSaneExtendedWidget::Mode mode, const QString &format);
    QString currentTextFormat(KSaneExtendedWidget::Mode mode);

    void setCurrentMultipageFormat(KSaneExtendedWidget::Mode mode, const QString &format);
    QString currentMultipageFormat(KSaneExtendedWidget::Mode mode);

    QString currentFormat(KSaneExtendedWidget::Mode mode);

    KSaneExtendedWidget::SaneStatus saneStatus();

public Q_SLOTS:
    /** This method can be used to cancel a scan or prevent an automatic new scan. */
    void scanCancel();

    /** This method can be used to start a scan (if no GUI is needed).
    * @note libksane may return one or more images as a result of one invocation of this slot.
    * If no more images are wanted scanCancel should be called in the slot handling the
    * imageReady signal. */
    void scanFinal();

    void scanMultipage();

    /** This method can be used to start a preview scan. */
    void startPreviewScan();

    void sendUserMessage(int type, const QString &strStatus);

    void recognizePage(const QSharedPointer<KSanePage> &page);


Q_SIGNALS:
    /**
     * This Signal is emitted when a final scan is ready.
     * @deprecated since 21.08
     * @see use new scannedImageReady(const QImage &scannedImage) instead
     * @param data is the byte data containing the image.
     * @param width is the width of the image in pixels.
     * @param height is the height of the image in pixels.
     * @param bytes_per_line is the number of bytes used per line. This might include padding
     * and is probably only relevant for 'FormatBlackWhite'.
     * @param format is the KSane image format of the data. */
    KSANE_DEPRECATED void imageReady(QByteArray &data, int width, int height,
                    int bytes_per_line, int format);

    /**
     * This signal is emitted when the scanning has ended.
     * @param status contains a ScanStatus status code.
     * @param strStatus If an error has occurred this string will contain an error message.
     * otherwise the string is empty. */
    void scanDone(int status, const QString &strStatus);

    /**
     * This signal is emitted every time the device list is updated or
     * after initGetDeviceList() is called.
     * @param deviceList is a QList of KSaneWidget::DeviceInfo that contain the
     * device name, model, vendor and type of the attached scanners.
     * @note The list is only a snapshot of the current available devices. Devices
     * might be added or removed/opened after the signal is emitted.
     */
    void availableDevices(const QList<KSaneExtendedWidget::DeviceInfo> &deviceList);

    /**
     * This Signal is emitted when a hardware button is pressed.
     * @param optionName is the untranslated technical name of the sane-option.
     * @param optionLabel is the translated user visible label of the sane-option.
     * @param pressed indicates if the value is true or false.
     * @note The SANE standard does not specify hardware buttons and their behaviors,
     * so this signal is emitted for sane-options that behave like hardware buttons.
     * That is the sane-options are read-only and type boolean. The naming of hardware
     * buttons also differ from backend to backend.
     */
    void buttonPressed(const QString &optionName, const QString &optionLabel, bool pressed);

    /**
     * This signal is emitted when the user is to be notified about something.
     * @note If no slot is connected to this signal the message will be displayed in a KMessageBox.
     * @param type contains a ScanStatus code to identify the type of message (error/info/...).
     * @param strStatus If an error has occurred this string will contain an error message.
     * otherwise the string is empty. */
    void userMessage(int type, const QString &strStatus, KSaneExtendedWidget::SaneStatus saneStatus);

    /**
     * This Signal is emitted for progress information during a scan.
     * The GUI already has a progress bar, but if the GUI is hidden,
     * this can be used to display a progress bar.
     * @param percent is the percentage of the scan progress (0-100). */
    void scanProgress(int percent);

    void multipageChanged(bool multipage);
    void savePage(const KSanePagePointer &page);
    void savePages(const QList<KSanePagePointer> &pages);
    void recognizePages(const QList<KSanePagePointer> &pages);
    void openDir(const QString &dir);

    void changeDevice();

    void busyChanged(bool busy);

    void showActivityFrame();
    void hideActivityFrame();
    void showWarmingUp();
    void hideWarmingUp();
    void progressChanged(int value);

    void scanStarted(bool isMultipage, bool isOcr);

    void modeChanged(KSaneExtendedWidget::Mode mode);

    void savePathChanged(const QString &path);

    void currentFormatChanged(const QString &format);

private:
    KSaneExtendedWidgetPrivate *m_d;
};

}

#endif // KSANEEXTENDEDWIDGET_H
